@@ -3,7 +3,7 @@
 //  RQuantLib -- R interface to the QuantLib libraries
 //
 //  Copyright (C) 2005 - 2007  Dominick Samperi
-//  Copyright (C) 2007 - 2014  Dirk Eddelbuettel
+//  Copyright (C) 2007 - 2016  Dirk Eddelbuettel
 //
 //  This file is part of RQuantLib.
 //
@@ -35,11 +35,25 @@ ObservableDB::ObservableDB() {
     db_["d1y"] = new RQLObservable(RQLDeposit, 1, 0, QuantLib::Years);
     db_["s2y"] = new RQLObservable(RQLSwap, 2, 0, QuantLib::Years);
     db_["s3y"] = new RQLObservable(RQLSwap, 3, 0, QuantLib::Years);
+    db_["s4y"] = new RQLObservable(RQLSwap, 4, 0, QuantLib::Years);
     db_["s5y"] = new RQLObservable(RQLSwap, 5, 0, QuantLib::Years);
+    db_["s6y"] = new RQLObservable(RQLSwap, 6, 0, QuantLib::Years);
+    db_["s7y"] = new RQLObservable(RQLSwap, 7, 0, QuantLib::Years);
+    db_["s8y"] = new RQLObservable(RQLSwap, 8, 0, QuantLib::Years);
+    db_["s9y"] = new RQLObservable(RQLSwap, 9, 0, QuantLib::Years);
     db_["s10y"] = new RQLObservable(RQLSwap, 10, 0, QuantLib::Years);
+    db_["s12y"] = new RQLObservable(RQLSwap, 12, 0, QuantLib::Years);
     db_["s15y"] = new RQLObservable(RQLSwap, 15, 0, QuantLib::Years);
     db_["s20y"] = new RQLObservable(RQLSwap, 20, 0, QuantLib::Years);
+    db_["s25y"] = new RQLObservable(RQLSwap, 25, 0, QuantLib::Years);
     db_["s30y"] = new RQLObservable(RQLSwap, 30, 0, QuantLib::Years);
+    db_["s40y"] = new RQLObservable(RQLSwap, 40, 0, QuantLib::Years);
+    db_["s50y"] = new RQLObservable(RQLSwap, 50, 0, QuantLib::Years);
+    db_["s60y"] = new RQLObservable(RQLSwap, 60, 0, QuantLib::Years);
+    db_["s70y"] = new RQLObservable(RQLSwap, 70, 0, QuantLib::Years);
+    db_["s80y"] = new RQLObservable(RQLSwap, 80, 0, QuantLib::Years);
+    db_["s90y"] = new RQLObservable(RQLSwap, 90, 0, QuantLib::Years);
+    db_["s100y"] = new RQLObservable(RQLSwap, 100, 0, QuantLib::Years);
     db_["fut1"] = new RQLObservable(RQLFuture, 1, 0, QuantLib::Months);
     db_["fut2"] = new RQLObservable(RQLFuture, 2, 0, QuantLib::Months);
     db_["fut3"] = new RQLObservable(RQLFuture, 3, 0, QuantLib::Months);
@@ -57,6 +71,78 @@ ObservableDB::ObservableDB() {
 // Get RateHelper used to build the yield curve corresponding to a
 // database key ('ticker') and observed rate/price.
 boost::shared_ptr<QuantLib::RateHelper> 
+ObservableDB::getRateHelper(std::string& ticker, QuantLib::Rate r, double fixDayCount,
+                            double fixFreq, int floatFreq) {
+    RQLMapIterator iter = db_.find(ticker);
+    if (iter == db_.end()) {
+        std::string errortxt = "Unknown curve construction instrument: " + ticker;
+        Rcpp::stop(errortxt);
+    }
+    RQLObservable *p = iter->second;
+    RQLObservableType type = p->getType();
+    int n1 = p->getN1(), n2 = p->getN2();
+    QuantLib::TimeUnit units = p->getUnits();
+    
+    QuantLib::Date settlementDate = RQLContext::instance().settleDate;
+    QuantLib::Calendar calendar = RQLContext::instance().calendar;
+    QuantLib::Integer fixingDays = RQLContext::instance().fixingDays;
+    QuantLib::DayCounter depositDayCounter = QuantLib::Actual360();
+    
+    // Tried to use a switch statement here, but there was an
+    // internal compiler error using g++ Version 3.2.2.
+    if (type == RQLDeposit) {
+        boost::shared_ptr<QuantLib::Quote> quote(new QuantLib::SimpleQuote(r));
+        boost::shared_ptr<QuantLib::RateHelper> 
+            depo(new QuantLib::DepositRateHelper(QuantLib::Handle<QuantLib::Quote>(quote),
+                                                 n1*units, fixingDays,      
+                                                 calendar, QuantLib::ModifiedFollowing, 
+                                                 true, /*fixingDays,*/ depositDayCounter));
+        return depo;
+    } else if (type == RQLSwap) {
+        QuantLib::Frequency swFixedLegFrequency = getFrequency(fixFreq);
+        QuantLib::BusinessDayConvention swFixedLegConvention = QuantLib::Unadjusted;
+        QuantLib::DayCounter swFixedLegDayCounter = getDayCounter(fixDayCount);
+        boost::shared_ptr<QuantLib::IborIndex> swFloatingLegIndex(new QuantLib::Euribor(QuantLib::Period(floatFreq,QuantLib::Months)));
+        boost::shared_ptr<QuantLib::Quote> quote(new QuantLib::SimpleQuote(r));
+        boost::shared_ptr<QuantLib::RateHelper> 
+            swap(new QuantLib::SwapRateHelper(QuantLib::Handle<QuantLib::Quote>(quote),
+                                              n1*QuantLib::Years, /*fixingDays,*/
+                                              calendar, swFixedLegFrequency,
+                                              swFixedLegConvention, swFixedLegDayCounter,
+                                              swFloatingLegIndex));
+        return swap;
+    } else if (type == RQLFuture) {
+        QuantLib::Integer futMonths = 3;
+        QuantLib::Date imm = QuantLib::IMM::nextDate(settlementDate);
+        for (int i = 1; i < n1; i++)
+            imm = QuantLib::IMM::nextDate(imm+1);
+        //Rcpp::Rcout << "Curves: IMM Date is " << imm << std::endl;
+        boost::shared_ptr<QuantLib::Quote> quote(new QuantLib::SimpleQuote(r));
+        boost::shared_ptr<QuantLib::RateHelper> 
+            future(new QuantLib::FuturesRateHelper(QuantLib::Handle<QuantLib::Quote>(quote),
+                                                   imm, futMonths, calendar, 
+                                                   QuantLib::ModifiedFollowing,
+                                                   true, // added bool endOfMonth variable
+                                                   depositDayCounter));
+        return future;
+    } else if (type == RQLFRA) {
+        boost::shared_ptr<QuantLib::Quote> quote(new QuantLib::SimpleQuote(r));
+        boost::shared_ptr<QuantLib::RateHelper> 
+            FRA(new QuantLib::FraRateHelper(QuantLib::Handle<QuantLib::Quote>(quote),
+                                            n1, n2, fixingDays, calendar, 
+                                            QuantLib::ModifiedFollowing,
+                                            true, /*fixingDays,*/ depositDayCounter));
+        return FRA;
+    } else {
+        Rcpp::stop("Bad type in curve construction");
+    }
+    // not reached
+    boost::shared_ptr<QuantLib::RateHelper> tmp;
+    return tmp;
+}
+
+//*** original rate helper, kept to ensure Bermudan works. Change in future - Terry Leitch 16 Mar '16
+boost::shared_ptr<QuantLib::RateHelper> 
 ObservableDB::getRateHelper(std::string& ticker, QuantLib::Rate r) {
     RQLMapIterator iter = db_.find(ticker);
     if (iter == db_.end()) {
@@ -72,14 +158,14 @@ ObservableDB::getRateHelper(std::string& ticker, QuantLib::Rate r) {
     QuantLib::Calendar calendar = RQLContext::instance().calendar;
     QuantLib::Integer fixingDays = RQLContext::instance().fixingDays;
     QuantLib::DayCounter depositDayCounter = QuantLib::Actual360();
-
+    
     // Tried to use a switch statement here, but there was an
     // internal compiler error using g++ Version 3.2.2.
     if (type == RQLDeposit) {
         boost::shared_ptr<QuantLib::Quote> quote(new QuantLib::SimpleQuote(r));
         boost::shared_ptr<QuantLib::RateHelper> 
             depo(new QuantLib::DepositRateHelper(QuantLib::Handle<QuantLib::Quote>(quote),
-                                                 n1*units, fixingDays,	
+                                                 n1*units, fixingDays,      
                                                  calendar, QuantLib::ModifiedFollowing, 
                                                  true, /*fixingDays,*/ depositDayCounter));
         return depo;
@@ -126,6 +212,8 @@ ObservableDB::getRateHelper(std::string& ticker, QuantLib::Rate r) {
     return tmp;
 }
 
+
+
 // Return the term structure built using a set of RateHelpers (curveInput)
 // employing the specified interpolation method and day counter.
 boost::shared_ptr<QuantLib::YieldTermStructure> 
@@ -133,90 +221,90 @@ getTermStructure (std::string& interpWhat, std::string& interpHow,
                   const QuantLib::Date& settlementDate,
                   const std::vector<boost::shared_ptr<QuantLib::RateHelper> >& curveInput,
                   QuantLib::DayCounter& dayCounter, QuantLib::Real tolerance) {
-
+    
     // the identifiers are just too bloody long so this functions like a 120 col or so display
     
     if (interpWhat.compare("discount") == 0 && interpHow.compare("linear") == 0) {
         boost::shared_ptr<QuantLib::YieldTermStructure> 
             ts(new QuantLib::PiecewiseYieldCurve<QuantLib::Discount, 
-                                                 QuantLib::Linear>(settlementDate, 
-                                                                   curveInput, dayCounter, 
-                                                                   std::vector<QuantLib::Handle<QuantLib::Quote> >(),
-                                                                   std::vector<QuantLib::Date>(),
-                                                                   tolerance));
+               QuantLib::Linear>(settlementDate, 
+                                 curveInput, dayCounter, 
+                                 std::vector<QuantLib::Handle<QuantLib::Quote> >(),
+                                 std::vector<QuantLib::Date>(),
+                                 tolerance));
         return ts;
     } else if (interpWhat.compare("discount") == 0 && interpHow.compare("loglinear") == 0) {
         boost::shared_ptr<QuantLib::YieldTermStructure> 
             ts(new QuantLib::PiecewiseYieldCurve<QuantLib::Discount,
-                                                 QuantLib::LogLinear>(settlementDate, 
-                                                                      curveInput, dayCounter, 
-                                                                      std::vector<QuantLib::Handle<QuantLib::Quote> >(),
-                                                                      std::vector<QuantLib::Date>(),
-                                                                      tolerance));
+               QuantLib::LogLinear>(settlementDate, 
+                                    curveInput, dayCounter, 
+                                    std::vector<QuantLib::Handle<QuantLib::Quote> >(),
+                                    std::vector<QuantLib::Date>(),
+                                    tolerance));
         return ts;
     } else if (interpWhat.compare("discount") == 0 && interpHow.compare("spline") == 0) {
         boost::shared_ptr<QuantLib::YieldTermStructure> 
             ts(new QuantLib::PiecewiseYieldCurve<QuantLib::Discount, 
-                                                 QuantLib::Cubic>(settlementDate, 
-                                                                  curveInput, dayCounter, 
-                                                                  std::vector<QuantLib::Handle<QuantLib::Quote> >(),
-                                                                  std::vector<QuantLib::Date>(),
-                                                                  tolerance));
+               QuantLib::Cubic>(settlementDate, 
+                                curveInput, dayCounter, 
+                                std::vector<QuantLib::Handle<QuantLib::Quote> >(),
+                                std::vector<QuantLib::Date>(),
+                                tolerance));
         return ts;
     } else if (interpWhat.compare("forward") == 0 &&
-              interpHow.compare("linear") == 0) {
+               interpHow.compare("linear") == 0) {
         boost::shared_ptr<QuantLib::YieldTermStructure> 
             ts(new QuantLib::PiecewiseYieldCurve<QuantLib::ForwardRate, 
-                                                 QuantLib::Linear>(settlementDate, 
-                                                                   curveInput, dayCounter, 
-                                                                   std::vector<QuantLib::Handle<QuantLib::Quote> >(),
-                                                                   std::vector<QuantLib::Date>(),
-                                                                   tolerance));
+               QuantLib::Linear>(settlementDate, 
+                                 curveInput, dayCounter, 
+                                 std::vector<QuantLib::Handle<QuantLib::Quote> >(),
+                                 std::vector<QuantLib::Date>(),
+                                 tolerance));
         return ts;
     } else if (interpWhat.compare("forward") == 0 && interpHow.compare("loglinear") == 0) {
         boost::shared_ptr<QuantLib::YieldTermStructure> 
             ts(new QuantLib::PiecewiseYieldCurve<QuantLib::ForwardRate,
-                                                 QuantLib::LogLinear>(settlementDate, 
-                                                                      curveInput, dayCounter, 
-                                                                      std::vector<QuantLib::Handle<QuantLib::Quote> >(),
-                                                                      std::vector<QuantLib::Date>(),
-                                                                      tolerance));
+               QuantLib::LogLinear>(settlementDate, 
+                                    curveInput, dayCounter, 
+                                    std::vector<QuantLib::Handle<QuantLib::Quote> >(),
+                                    std::vector<QuantLib::Date>(),
+                                    tolerance));
         return ts;
     } else if (interpWhat.compare("forward") == 0 && interpHow.compare("spline") == 0) {
         boost::shared_ptr<QuantLib::YieldTermStructure> 
             ts(new QuantLib::PiecewiseYieldCurve<QuantLib::ForwardRate,
-                                                 QuantLib::Cubic>(settlementDate, 
-                                                                  curveInput, dayCounter, 
-                                                                  std::vector<QuantLib::Handle<QuantLib::Quote> >(),
-                                                                  std::vector<QuantLib::Date>(),
-                                                                  tolerance));
+               QuantLib::Cubic>(settlementDate, 
+                                curveInput, dayCounter, 
+                                std::vector<QuantLib::Handle<QuantLib::Quote> >(),
+                                std::vector<QuantLib::Date>(),
+                                tolerance));
         return ts;
     } else if (interpWhat.compare("zero") == 0 && interpHow.compare("linear") == 0) {
         boost::shared_ptr<QuantLib::YieldTermStructure> 
             ts(new QuantLib::PiecewiseYieldCurve<QuantLib::ZeroYield,
-                                                 QuantLib::Linear>(settlementDate, 
-                                                                   curveInput, dayCounter, 
-                                                                   std::vector<QuantLib::Handle<QuantLib::Quote> >(),
-                                                                   std::vector<QuantLib::Date>(),
-                                                                   tolerance));
+               QuantLib::Linear>(settlementDate, 
+                                 curveInput, dayCounter, 
+                                 std::vector<QuantLib::Handle<QuantLib::Quote> >(),
+                                 std::vector<QuantLib::Date>(),
+                                 tolerance));
         return ts;
     } else if (interpWhat.compare("zero") == 0 && interpHow.compare("loglinear") == 0) {
         boost::shared_ptr<QuantLib::YieldTermStructure> 
             ts(new QuantLib::PiecewiseYieldCurve<QuantLib::ZeroYield,
-                                                 QuantLib::LogLinear>(settlementDate, 
-                                                                      curveInput, dayCounter, 
-                                                                      std::vector<QuantLib::Handle<QuantLib::Quote> >(),
-                                                                      std::vector<QuantLib::Date>(),
-                                                                      tolerance));
+               QuantLib::LogLinear>(settlementDate, 
+                                    curveInput, dayCounter, 
+                                    std::vector<QuantLib::Handle<QuantLib::Quote> >(),
+                                    std::vector<QuantLib::Date>(),
+                                    tolerance));
         return ts;
     } else if (interpWhat.compare("zero") == 0 && interpHow.compare("spline") == 0) {
         boost::shared_ptr<QuantLib::YieldTermStructure> 
             ts(new QuantLib::PiecewiseYieldCurve<QuantLib::ZeroYield,
-                                                 QuantLib::Cubic>(settlementDate, 
-                                                                  curveInput, dayCounter, 
-                                                                  std::vector<QuantLib::Handle<QuantLib::Quote> >(),
-                                                                  std::vector<QuantLib::Date>(),
-                                                                  tolerance));
+               QuantLib::Cubic>(settlementDate, 
+                                curveInput, dayCounter, 
+                                std::vector<QuantLib::Handle<QuantLib::Quote> >(),
+                                std::vector<QuantLib::Date>(),
+                                tolerance));
         return ts;
     } else {
         Rcpp::Rcout << "interpWhat = " << interpWhat << std::endl;
